/*
  Programa .COM para ser ejecutado en un entorno DOS
  Este programa muestra el uso de lenguaje ensamblador para interactuar
  con el Entorno de Ejecucion de DOS. Este entorno de ejecucion se
  puede encontrar en sistemas operativos DOS y Windows. En sistemas
  operativos diferentes a DOS o Windows, se puede utilizar DosBox para
  ejecutar archivos COM.

  Los programas COM tienen las siguientes caracteristicas:
  * Se ejecutan en un entorno de 16 bits, similar al modo real
  * No pueden ocupar mas de un segmento en modo real (64 KB)
  * Son cargados en la direccion 0x100 de un segmento (los primeros 256 bytes
  * se reservan al cargador)
  * Pueden usar los servicios ofrecidos por DOS, en especial
  * los servicios ofrecidos por medio de la Interrupcion 21 (INT 0x21)

  El programa hace uso de los servicios de DOS para:
  * Imprimir un mensaje de bienvenida por pantalla
  * Imprimir un mensaje de despedida
  * Esperar a que se pulse una tecla
  * Terminar su ejecucion.
*/

.code16 /* codigo de 16 bits*/
.section .data /* Seccion de datos*/
variable:	/* Declaracion de una variable, de tipo word, con valor = 0 */
 .word 0x0000
greeting: /* Declaracion de una cadena de tipo asciz (ascii terminada en nulo)*/
 .asciz "Hello from GNU Assembler\n$"  /* La funcion para imprimir cadenas
 										  en DOS requiere que la cadena termine
 										  en '$'*/
linea:	/* Buffer de salida*/
  .space 80

goodbye: /* Otra cadena de caracteres */
 .asciz "Bye from GNU Assembler. Press any key to continue.\n$"

/* Seccion de texto (codigo) del programa */
.section .text

.globl start /* directiva necesaria para el linker */
start:	/* Simbolo (etiqueta) en el cual comienza la ejecucion */

  /* Imprimir un string en la pantalla usando los servicios de DOS (INT 21,9)
     %ah = 0x09 = Funcion de DOS (print string)
     %dx = direccion de memoria de la cadena a imprimir

  */

  movb $0x09, %ah	/* Almacenar 0x09 a %ax */
  movw $greeting, %dx /* Direccion de memoria de la cadena a imprimir en %dx*/
  int $0x21			/* Llamada a los servicios de DOS */
  

  /* Imprimir otro string en la pantalla */
  /* movb $0x09, %ah*/ /* se supone que no ha cambiado el valor de %ah*/
  movw $goodbye, %dx  /* Direccion de memoria de la cadena a imprimir*/
  int $0x21 /* Llamada a los servicios de DOS */

  /* Leer una tecla usando los servicios de DOS (INT 21,1)
    %ah = 0x01 = Funcion de DOS (Keyboard input with echo)

    Retorna en %al el caracter leido. Si se presiono una tecla especial,
    retorna 0 en %al y se debe repetir la lectura para obtener la tecla
  */

  movb $0x01, %ah
  int $0x21

  /* Por ultimo, saltar al simbolo 'finished' */
  jmp finished


printhex:
  pushw %bp
  movw %sp, %bp
  subw $2, %sp /* Reservar espacio para una variable*/

  pusha

  movw 4(%bp), %ax
  movw %ax, -2(%bp)
  xorw %bx, %bx

  movw $4, %cx

 printhex_loop:
  movw -2(%bp), %ax
  rolw $4, %ax
  movw %ax, -2(%bp)
  andw $0x000F, %ax
  cmpb $0x09, %al
  jle continue1
  subb $10, %al
  addb $0x41, %al
  jmp continue2

continue1:
  addb $0x30, %al
continue2:
  movb %al, linea(%bx)
  incw %bx
  loop printhex_loop
  movb $0x0A, linea(%bx)
  incw %bx
  movb $'$', linea(%bx)

  movb $0x09, %ah
  movw $linea, %dx
  int $0x21

  popa
  movw %bp, %sp
  popw %bp
  ret


finished:	/* Simbolo finished */

  /* Terminar el programa haciendo uso de los servicios de DOS (INT 21, 4C)
	ah = 0x4C
	Este es el metodo recomendado. Para versiones de DOS anteriores a 2.1,
	se debe utilizar INT 21, 0 o INT 20 para terminar.
  */
  movb $0x4C, %ah
  int $0x21

stack:
  .space 0x200
