/*
  Programa .COM para ser ejecutado en un entorno DOS
  Este programa muestra el uso de lenguaje ensamblador para interactuar
  con el Entorno de Ejecucion de DOS.

  El programa hace uso de los servicios de DOS para:
  * Imprimir un mensaje de bienvenida por pantalla
  * Esperar a que se pulse una tecla
  * Imprimir un mensaje de despedida
  * Esperar a que se pulse una tecla
  * Terminar su ejecucion.
*/

.intel_syntax noprefix /* Sintaxis Intel, evitar los prefijos en los registros*/
.code16
.section .data /* Seccion de datos*/
variable:	/* Declaracion de una variable, word (2 bytes), con valor = 0 */
 .word 0x0000
greeting: /* Declaracion de una cadena de tipo asciz (ascii terminada en nulo)*/
 .asciz "Hello from program. Press any key to continue.\n$"
         /* La funcion para imprimir cadenas en DOS requiere que la cadena
         termine en '$' */

goodbye: /* Otra cadena de caracteres */
 .asciz "Bye from program. Press any key to exit.\n$"

/* Seccion de texto (codigo) del programa */
.section .text

.global start /* Directiva necesaria para el linker */
start:	/* Simbolo (etiqueta) en el cual comienza la ejecucion */

  /* Imprimir un string en la pantalla usando los servicios de DOS (INT 21,9)
     %ah = 0x09 = Funcion de DOS (print string)
     %dx = direccion de memoria de la cadena a imprimir
  */

  mov ah, 0x09 /* ah = 0x09 */
  /* Direccion de memoria (desplazamiento) de la cadena a imprimir en %dx*/
  mov dx, OFFSET greeting /* dx = direccion de memoria de 'greeting' */

  int 0x21 /* Llamada al servicio de DOS */


  /* Leer una tecla usando los servicios de DOS (INT 21,1)
    %ah = 0x01 = Funcion de DOS (Keyboard input with echo)

    Retorna en %al el caracter leido. Si se presiono una tecla especial,
    retorna 0 en %al y se debe repetir la lectura para obtener la tecla
  */

  movb ah, 0x01 /* ah = 0x01 */
  int 0x21 /* Llamada al servicio de DOS */

  /* Por ultimo, saltar al simbolo 'finished' */
  jmp finished


finished:	/* Simbolo finished */

  /* Imprimir otro string en la pantalla */
  mov ah, 0x09 /* ah = 0x09 */
  /* Direccion de memoria (desplazamiento) de la cadena a imprimir*/
  movw dx, OFFSET goodbye

  int 0x21 /* Llamada al servicio de DOS */

  /* Leer de nuevo una tecla antes de terminar */
  movb ah, 0x01	/* ah = 0x01 */
  int 0x21 /* Llamada al servicio de DOS */

  /* Terminar el programa haciendo uso de los servicios de DOS (INT 21, 4C)
	ah = 0x4C
	Este es el metodo recomendado. Para versiones de DOS anteriores a 2.1,
	se debe utilizar INT 21, 0 o INT 20 para terminar.s
  */
  movb ah, 0x4C /* ah = 0x4C */
  int 0x21 /* Llamada al servicio de DOS */
