/*
  Programa .COM para ser ejecutado en un entorno DOS
  Este programa muestra como crear y utilizar rutinas

  El programa hace uso de rutinas que usan los servicios de DOS para:
  * Imprimir un mensaje de bienvenida por pantalla
  * Imprimir un caracter
  * Esperar a que se pulse una tecla
  * Imprimir un mensaje de despedida
  * Esperar a que se pulse una tecla
  * Terminar su ejecucion.

Consulte la documentacion en la cual se explica la implementacion de rutinas
en lenguaje ensamblador de 16 bits.

  objdump -S program.o -M intel,i8086
*/

.intel_syntax noprefix /* Sintaxis Intel, evitar los prefijos de los registros*/
.code16
.section .data /* Seccion de datos*/
variable:	/* Declaracion de una variable, de tipo word, con valor = 0 */
 .word 0x0000
greeting: /* Declaracion de una cadena de tipo asciz (ascii terminada en nulo)*/
 .asciz "Hello from program. Press any key to continue!.\n$"
 /* La funcion para imprimir cadenas en DOS requiere que la cadena termine
 	en '$'*/

linea:	/* Buffer de salida*/
  .space 80

goodbye: /* Otra cadena de caracteres */
 .asciz "Bye from program. Press any key to exit.\n$"

/* Seccion de texto (codigo) del programa */
.section .text

.global start /* directiva necesaria para el linker */
start:	/* Simbolo (etiqueta) en el cual comienza la ejecucion */

  /* Invocar a la rutina printstring
  pasando como parametro en la pila la direccion de memoria de la cadena
  a imprimir */

  push OFFSET greeting
  call printstring
  add sp, 2

  /* Invocar a la rutina printchar, pasando como parametro en la
  pila el caracter a imprimir */

  push '>'
  call printchar
  add sp, 2

  /* Llamar la rutina getkey. Esta rutina no recibe parametros */
  call getkey

  /* Por ultimo, saltar al simbolo 'finished' */
  jmp finished

finished:	/* Simbolo finished */

  /* Imprimir otro string en la pantalla haciendo uso de la rutina*/
  push OFFSET goodbye  /* Direccion de memoria de la cadena a imprimir*/
  call printstring
  add sp, 2

 /* Invocar de nuevo la rutina para leer una tecla*/
  call getkey

 /* Invocar la rutina para terminar el programa */
  call terminate
 /* El procesador no deberia retornar de la rutina terminate.*/


 /* Implementacion de rutinas*/

 /*
  Rutina: terminate
  Descripcion: Termina el programa correctamente usando los servicios de DOS.
  Entrada:
  	Ninguno
  Salida:
  	Ninguno. Al ejecutarse esta rutina el programa termina.
  	Esta funcion no retorna, ya que se invoca el servicio de DOS para terminar
  	el programa.
 */
 terminate:
   /* Terminar el programa haciendo uso de los servicios de DOS (INT 21, 4C)
	ah = 0x4C
	Este es el metodo recomendado. Para versiones de DOS anteriores a 2.1,
	se debe utilizar INT 21, 0 o INT 20 para terminar.
  */
  mov ah, 0x4C	/* ah = 0x4C */
  int 0x21		/* Llamada al servicio de DOS */
  /* Esta funcion no deberia retornar!. De todas maneras retornar,
  	 por si algo sucede, incluir 'ret'  para que la definicion
  	 de la rutina siga la convencion estandar. */
  ret

#include "stdio.S" /* Inclur las rutinas de entrada / salida */
