/* Este archivo contiene algunas rutinas basicas para gestionar
la entrada y la salida usando los servicios de la BIOS */

/*
Rutina: printchar
Descripcion: Esta rutina usa los servicios de DOS para imprimir un caracter en
la salida estandar (la pantalla)
Entrada:
	[bp + 4]: El byte menos significativo contiene el caracter a imprimir
Salida:
	Ninguno.
*/
printchar:
	push bp /* Crear el marco de pila */
	mov bp, sp

  /* Imprimir el caracter en la pantalla usando los servicios de DOS (INT 21,2)
     %ah = 0x02 = Funcion de DOS (display output)
     %dx = direccion de memoria de la cadena a imprimir
  */

	mov dl, BYTE PTR [bp + 4] /* Recuperar el primer parametro (un byte) */
	mov ah, 0x02	/* ah = 0x02 */
	int 0x21		/* Llamar al servicio de DOS*/

	mov sp, bp /* Cerrar el marco de pila*/
	pop bp
	ret	/* Retornar */


/*
Rutina: printstring
Descripcion: Esta rutina usa los servicios de DOS para imprimir una cadena de
caracteres
Entrada:
	[bp + 4]: Direccion de memoria de la cadena terminada en '$' a imprimir
Salida:
	Ninguno.
*/
printstring:
	push bp	/* Crear el marco de pila*/
	mov bp, sp

	/* Imprimir la cadena en la pantalla usando los servicios de DOS (INT 21,9)
     %ah = 0x09 = Funcion de DOS (print string)
     %dx = direccion de memoria de la cadena a imprimir
  */
	mov dx, WORD PTR [bp + 4] /* Recuperar el parametro de la pila */
	mov ah, 0x09
	int 0x21

	mov sp, bp	/* Cerrar el marco de pila*/
	pop bp
	ret			/* Retornar de la rutina*/

/*
 Rutina: getkey
 Descripcion: Esta rutina lee una tecla haciendo uso de los servicios de DOS
 Entrada:
 	Ninguno
 Salida:
 	al contiene la tecla leida
*/
getkey:
/* Leer una tecla usando los servicios de DOS (INT 21,1)
    %ah = 0x01 = Funcion de DOS (Keyboard input with echo)

    Retorna en %al el caracter leido. Si se presiono una tecla especial,
    retorna 0 en %al y se debe repetir la lectura para obtener la tecla
  */

  mov ah, 0x01	/* ah = 0x01 */
  int 0x21		/* Llamada al servicio de DOS*/

  ret			/* Retornar de la rutina*/

