/*
Programa .COM para ser ejecutado en un entorno DOS

Este programa implementa una serie de rutinas para manipular cadenas de
caracteres.

Consulte la documentacion en la cual se explica la implementacion de rutinas
en lenguaje ensamblador de 16 bits.

  objdump -S program.o -M intel,i8086
*/

.intel_syntax noprefix /* Sintaxis Intel, evitar los prefijos de los registros*/
.code16
.section .data /* Seccion de datos*/
variable:	/* Declaracion de una variable, de tipo word, con valor = 0 */
 .word 0x0000
greeting: /* Declaracion de una cadena de tipo asciz (ascii terminada en nulo)*/
 .asciz "My length is: $"
 /* La funcion para imprimir cadenas en DOS requiere que la cadena termine
 	en '$'*/

greeting_copy:
	.asciz "MY LENGTH IS: $"

linea:	/* Buffer de salida*/
  .space 80

goodbye: /* Otra cadena de caracteres */
 .asciz "Bye from program. Press any key to exit.\n$"

/* Seccion de texto (codigo) del programa */
.section .text

.global start /* directiva necesaria para el linker */
start:	/* Simbolo (etiqueta) en el cual comienza la ejecucion */

  /* printstring(greeting) */
  push OFFSET greeting
  call printstring
  add sp, 2

  /* ax = strlen(greeting) */
  push OFFSET greeting
  call strlen
  add sp, 2

  /* printdec(ax) */
  push ax
  call printdec
  add sp, 2

  /* ax = strcmp (greeting, greeting_copy */
  push OFFSET greeting_copy
  push OFFSET greeting
  call strcmp
  add sp, 4

  /* printhex(ax)*/
  push ax
  call printhex
  add sp, 2

   /* strcpy(greeting, greeting_copy)*/
  push OFFSET greeting_copy
  push offset greeting
  call strcpy
  add sp, 4


  /* printstring(greeting) */
  push OFFSET greeting
  call printstring
  add sp, 2

  /* ax = strlen(greeting) */
  push OFFSET greeting
  call strlen
  add sp, 2

  /* printdec(ax) */
  push ax
  call printdec
  add sp, 2

  /* Por ultimo, saltar al simbolo 'finished' */
  jmp finished

finished:	/* Simbolo finished */

  /* printstring (goodbye) */
  push OFFSET goodbye  /* Direccion de memoria de la cadena a imprimir*/
  call printstring
  add sp, 2

 /* getkey() */
  call getkey

 /* terminate() */
  call terminate

 /* El procesador no deberia retornar de la rutina terminate.*/


 /* Implementacion de rutinas*/

 /*
  Rutina: terminate
  Descripcion: Termina el programa correctamente usando los servicios de DOS.
  Entrada:
  	Ninguno
  Salida:
  	Ninguno. Al ejecutarse esta rutina el programa termina.
  	Esta funcion no retorna, ya que se invoca el servicio de DOS para terminar
  	el programa.
 */
 terminate:
   /* Terminar el programa haciendo uso de los servicios de DOS (INT 21, 4C)
	ah = 0x4C
	Este es el metodo recomendado. Para versiones de DOS anteriores a 2.1,
	se debe utilizar INT 21, 0 o INT 20 para terminar.
  */
  mov ah, 0x4C	/* ah = 0x4C */
  int 0x21		/* Llamada al servicio de DOS */
  /* Esta funcion no deberia retornar!. De todas maneras retornar,
  	 por si algo sucede, incluir 'ret'  para que la definicion
  	 de la rutina siga la convencion estandar. */
  ret

/* Rutinas basicas para gestionar la entrada y la salida usando los servicios
de la BIOS */

/*
Rutina: printchar
Descripcion: Esta rutina usa los servicios de DOS para imprimir un caracter en
la salida estandar (la pantalla)
Entrada:
	[bp + 4]: El byte menos significativo contiene el caracter a imprimir
Salida:
	Ninguno.
*/
printchar:
	push bp /* Crear el marco de pila */
	mov bp, sp

  /* Imprimir el caracter en la pantalla usando los servicios de DOS (INT 21,2)
     %ah = 0x02 = Funcion de DOS (display output)
     %dx = direccion de memoria de la cadena a imprimir
  */

	mov dl, BYTE PTR [bp + 4] /* Recuperar el primer parametro (un byte) */
	mov ah, 0x02	/* ah = 0x02 */
	int 0x21		/* Llamar al servicio de DOS*/

	mov sp, bp /* Cerrar el marco de pila*/
	pop bp
	ret	/* Retornar */


/*
Rutina: printstring
Descripcion: Esta rutina usa los servicios de DOS para imprimir una cadena de
caracteres
Entrada:
	[bp + 4]: Direccion de memoria de la cadena terminada en '$' a imprimir
Salida:
	Ninguno.
*/
printstring:
	push bp	/* Crear el marco de pila*/
	mov bp, sp

	/* Imprimir la cadena en la pantalla usando los servicios de DOS (INT 21,9)
     %ah = 0x09 = Funcion de DOS (print string)
     %dx = direccion de memoria de la cadena a imprimir
  */
	mov dx, WORD PTR [bp + 4] /* Recuperar el parametro de la pila */
	mov ah, 0x09
	int 0x21

	mov sp, bp	/* Cerrar el marco de pila*/
	pop bp
	ret			/* Retornar de la rutina*/

/*
Rutina: printbin
Descripcion: Esta rutina imprime un numero en formato binario.
Entrada:
	[bp + 4]  Numero de 16 bits a imprimir en formato binario
Salida:
	Ninguno.
*/
printbin:

	push bp /* Crear el marco de pila */
	mov bp, sp
	sub sp, 2	/* Crear espacio para una variable local de tipo word*/

	pusha	/* Almacenar en la pila los valores de los registros de proposito
			general */

	mov ax, WORD PTR [bp + 4]	/* Recuperar el parametro de la pila */

	mov WORD PTR [bp - 2], ax /* Y almacenarlo en la variable local */

	mov cx, 15	/* Indice en cx, imprimie bits 15...1 */

	xor bx, bx

	printbindigit:
		bt ax, cx /*Tomar el ax[cx] y almacenarlo en carry flag (CF)*/
		jc continue1bin  /* Si CF = 1, saltar a la etiqueta continue1bin */
		mov BYTE PTR [linea + bx], '0'
		jmp continue2bin
		continue1bin:
			mov BYTE PTR [linea + bx], '1'
		continue2bin:
			inc bx /* De cualquier forma incremente el apuntador de la cadena */
	loop printbindigit		/* cx = cx - 1, saltar a 'printbindigit' si cx > 0

	/* El ciclo termina cuando cx = 0*/

	/* Se imprimieron los bits 15...1, falta el bit 0.
	   cx ya tiene el valor de 0, simplemente verificar el bit ax[cx] */

	bt ax, cx /*Tomar el bit ax[cx] y almacenarlo en carry flag (CF)*/
	jc continue11bin  /* Si CF = 1, saltar a la etiqueta continue11bin */
	mov BYTE PTR [linea + bx], '0'
	jmp continue22bin
	continue11bin:
	mov BYTE PTR [linea + bx], '1'
	continue22bin:
	inc bx /* De cualquier forma incremente el apuntador de la cadena */

	mov BYTE PTR [linea + bx], '\n' /* fin del ciclo, incluir \n en la cadena
	  									 de salida */
	inc bx
	mov BYTE PTR [linea + bx], '$'

	/* printstring(linea) */
	push OFFSET linea
	call printstring
	add sp, 2

	popa

	mov sp, bp /* Cerrar el marco de pila */
	pop bp

	ret	/* Retornar de la rutina */

/*
Rutina: printdec
Descripcion: Esta rutina imprime un numero en formato decimal. Para ello
divide sucesivamente en potencias de 10 para sacar cada digito decimal,
empezando por 10000, que es la maxima potencia de 10 que se puede tener en 16
bits (teniendo en cuenta que el numero mas grande en 16 bits es 65535)
Entrada:
	[bp + 4] Numero de 16 bits (word) a imprimir en formato decimal
Salida:
	Ninguno.
*/
printdec:
	push bp	/* Crear el marco de pila */
	mov bp, sp
	sub sp, 4 /* Reservar espacio para variables locales:
				[bp - 2]: Numero a imprimir en decimal */

	pusha	/* Almacenar en la pila los valores de los registros de proposito
			general */



	mov cx, WORD PTR [bp + 4] /* Recuperar el numero de la pila */
	mov WORD PTR [bp - 2], cx /* Y almacenarlo en la variable local */
	mov WORD PTR [bp - 4], 10000 /* El maximo numero en 16 bits es
									65535, la maxima potencia de 10 es 10000 */

	xor bx, bx	/* bx = 0*/

	printdecloop:
	xor dx, dx	/* dx = 0 */
	mov ax, WORD PTR [bp - 2] /* Numero a dividir en ax */
	mov cx, WORD PTR [bp - 4] /* Potencia de 10 */

	div cx	/* Dividir dx:ax en cx, ax = cociente, dx = residuo */

	mov WORD PTR [bp - 2], dx /* Almacenar el residuo, es el nuevo numero
								 por el cual se debe dividir la potencia de 10*/

	/* El valor almacenado en ax (cociente) es el digito decimal a imprimir
	   Como un digito decimal se encuentra entre 0 y 9,  el valor
	   se encuentra almacenado al, es decir en los bits menos significativos
	   de ax */

	/* Si el valor del digito almacenado en ax es cero y si el indice de la
	 linea (bx) no se ha avanzado, saltar. Esto evita imprimir ceros a la
	 izquierda. */

	or ax, ax
	jnz print_dec_digit

	or bx, bx
	jz next_dec_digit


	print_dec_digit:
	/* Agregar 0x30 (ASCII '0' a al) */
	add al, 0x30

	/* al ahora contiene el codigo ASCII del digito decimal */

	/* Almacenar el digito decimal en la linea de salida */
	mov BYTE PTR [ linea + bx], al
	inc bx

	next_dec_digit:

	/* Ahora actualizar  la potencia de 10, dividiendo la potencia
	actual entre 10 */
	xor dx, dx /* dx = 0 */
	mov ax, cx /* ax = cx = potencia de 10 actual*/

	mov cx, 10 /* cx = 10 */

	div cx /* dividir dx:ax en cx, ax = cociente (nueva potencia), dx=residuo */

	cmp ax, 0 /* El cociente es cero, terminar!*/
	jz endprintdecloop

	mov WORD PTR [bp - 4], ax


	jmp printdecloop

	endprintdecloop:

	mov BYTE PTR [linea + bx], '\n' /* fin del ciclo, incluir \n en la cadena
	  									 de salida */
	  inc bx					/* Avanzar el apuntador en la cadena de salida*/
	  mov BYTE PTR [ linea + bx], '$' /* Terminar correctamente la linea
	  									de salida */

	 /* printstring(linea) */
      push OFFSET linea
	  call printstring
	  add sp, 2

	popa					/* Recuperar los valores de los registros de
	  						   proposito  general */
	mov sp, bp			/* Cerrar el marco de pila */
	pop bp
	ret					/* Retornar*/

/*
Rutina: printhex
Descripcion: Esta rutina imprime un numero en formato hexadecimal. Para ello
toma cada 4 digitos binarios, y encuentra el digito hexa correspondiente.
Entrada:
	[bp + 4] Numero de 16 bits (word) a imprimir en formato hexa
Salida:
	Nunguno.
*/

printhex:
	 push bp	/* Crear el marco de pila para la rutina */
	 mov bp, sp
	 sub sp, 2 /* Reservar espacio de dos bytes en la pila: una variable
	 			local (word) */

	 pusha	/* Almacenar en la pila los valores de los registros de proposito
			general */

	 mov ax, WORD PTR [bp + 4] /* Recuperar el primer parametro de la pila
	 							  (el numero a imprimir) */
	 mov WORD PTR [bp - 2], ax /* Y almacenarlo en la variable local */

	 xor bx, bx /* bx = 0, posicion en la cadena de salida  */

	 /* La arquitectura Intel es little-endian, lo que significa que en la parte
	 baja de los registros se almacenan los digitos menos significativos.
	 Sin embargo, para imprimir un numero hex se debe imprimir primero los
	 digitos mas significativos. Por esa razon, se debe 'rotar' hacia lia
	 izquierda el valor  almacenado en el registro, para que los bit mas
	 significativos pasen a ser los menos significativos. Este proceso se debe
	 repetir cuatro veces, ya que un numero binario de 16 bits tiene
	 4 digitos hex.
	 */

	  movw cx, 4 /* Numero de iteraciones en cx */

	 printhexdigit:	/* Ciclo para imprimir un digito hexa */

	  mov ax, WORD PTR [bp - 2] /* ax = [bp - 2] */
	  rol ax, 4					/* Rotar ax en 4 bits hacia la izquierda */
	  mov WORD PTR [bp - 2], ax /* [bp - 2] = ax  */
	  and ax, 0x000F			/* al contiene el valor del digito a imprimir */
	  cmpb al, 0x09				/* Comparar al con 9 */
	  jle continue1hex				/* si al <= 9, saltar a 'continue1' */
	  sub al, 10				/*  al  = al - 10 */
	  add al, 0x41				/* Agregar 0x41 (ascii 'A') al digito */
	  jmp continue2hex				/* Ir a finsi */

	continue1hex:
	  add al, 0x30				/* Agregar 0x30 (ascii '0') al digito */
	continue2hex:					/* finsi */
	  mov BYTE PTR [ linea + bx], al	/* Almacenar el caracter ascii del
	  									digito en la cadena de salida*/
	  inc bx					/* bx = bx + 1 . Avanzar el apuntador de la
	  							   linea de salida */
	  loop printhexdigit	/* cx = cx - 1, saltar a 'printhex digit' si cx > 0
	  							*/
	  mov BYTE PTR [linea + bx], '\n' /* fin del ciclo, incluir \n en la cadena
	  									 de salida */
	  inc bx					/* Avanzar el apuntador en la cadena de salida*/
	  mov BYTE PTR [ linea + bx], '$' /* Terminar correctamente la linea
	  									de salida */

	 /* printstring(linea) */
      push OFFSET linea
	  call printstring
	  add sp, 2

	  popa					/* Recuperar los valores de los registros de
	  						   proposito  general */
	  mov sp, bp			/* Cerrar el marco de pila */
	  pop bp
	  ret					/* Retornar*/

/*
 Rutina: getkey
 Descripcion: Esta rutina lee una tecla haciendo uso de los servicios de DOS
 Entrada:
 	Ninguno
 Salida:
 	al contiene la tecla leida
*/
getkey:
/* Leer una tecla usando los servicios de DOS (INT 21,1)
    %ah = 0x01 = Funcion de DOS (Keyboard input with echo)

    Retorna en %al el caracter leido. Si se presiono una tecla especial,
    retorna 0 en %al y se debe repetir la lectura para obtener la tecla
  */

  mov ah, 0x01	/* ah = 0x01 */
  int 0x21		/* Llamada al servicio de DOS*/

  ret			/* Retornar de la rutina*/

/* Rutinas para el manejo de cadenas de caracteres */

/*
Rutina: strlen
Descripcion: Calcula la longitud de una cadena terminada en '$' (o en nulo).
La maxima longitud de una cadena en 16 bits es 65535.
Entrada:
	[bp + 4]: Direccion de memoria de la cadena
Salida:
	ax : contiene la longitud de la cadena, en bytes
*/

strlen:
	push bp /* Crear el marco de pila */
	mov bp, sp
	sub sp, 2 /* Variable local (word) [bp - 2]: total : Numero de bytes*/

	pusha /* Registros de proposito general a la pila */
	pushf	/* Almacenar FLAGS en la pila */


	mov WORD PTR [bp - 2], 0 /* total  = 0*/

	mov si, WORD PTR [bp + 4] /* Parametro de entrada (direccion de
								 memoria de la cadena) en si */
	cld

	strlen_loop:
		lodsb	/* Almacenar un byte  de [si] en al, incrementar si */
		cmp al, '$'
		je end_strlen_loop
		cmp al, 0
		je end_strlen_loop
		inc WORD PTR [bp - 2] /* total = total + 1*/
	jmp strlen_loop

	end_strlen_loop:

	popf /* Recuperar FLAGS de la pila */
	popa /* Recuperar registros de proposito general */

	mov ax, WORD PTR [bp - 2] /* ax = total (valor de retorno) */

	mov sp, bp /* Cerrar el marco de pila */
	pop bp
	ret	/* Retornar de la rutina*/

/*
Rutina: strcpy
Descripcion:
	Permite copiar una cadena de caracteres a otra. Termina la copia
	cuando se encuentra el fin de cadena, representado por el caracter
	's' nulo.
Entrada:
	[bp + 4]: Direccion de la cadena de destino
	[bp + 6]: Direccion de la cadena fuente
Salida:
	ax contiene el numero de bytes copiados

*/
strcpy:
	push bp	/* Crear el marco de pila */
	mov bp, sp

	sub sp, 2 /* Variable local (total): # de caracteres copiados*/

	pusha /* Almacenar los valores de los registros de proposito general */
	pushf /* Almacenar el registro flags */

	cld /* Clear Direction Flag */

    mov di, WORD PTR [bp + 4]
    mov si, WORD PTR [bp + 6]

    mov WORD PTR [bp - 2], 0

	strcpy_loop:

		movsb
		inc WORD PTR [bp - 2] /* total = total + 1*/

		cmp WORD PTR [di - 1], '$'
		je strcpy_finished

		cmp WORD PTR [di - 1], 0
		jne strcpy_loop
		mov BYTE PTR [di - 1], '$'
		mov BYTE PTR [di], 0
		jmp strcpy_finished

	strcpy_finished:

	popf /* Recuperar el valor de flags */
	popa /* Recuperar el valor de los registros */
	mov ax, WORD PTR [bp - 2] /* Valor de retorno */

	mov sp, bp /* Cerrar el marco de pila */
	pop bp

	ret	/* Retornar de la rutina */


/*
Rutina: strcmp
Descripcion: Esta rutina compara dos cadenas de caracteres y determina si son
			 iguales.
Entrada:
	[bp + 4]: Cadena a comparar (destino)
	[bp + 6]: Cadena con la cual se desea comparar (fuente)
Salida:
	FLAGS contiene los valores requeridos para verificar
	con las instrucciones de salto condicional je, jne
*/
strcmp:
	push bp
	mov bp, sp

	sub sp, 2

	pusha

	mov di, WORD PTR [bp + 4]
	mov si, WORD PTR [bp + 6]

	mov WORD PTR [bp - 2], 0


	cld

	strcmp_loop:
		/* Primera cadena ya se termino? */
		mov al, BYTE PTR [si]
		mov ah, BYTE PTR [di]
		inc si
		inc di

		cmp al, '$' /* Se termino la fuente? */
		jne strcmp_test1 /* si no, saltar */

		cmp ah, '$' /* Tambien se termino el destino? */
		je strcmp_are_equal /* Las dos cadenas son iguales! */


		strcmp_test1:
		cmp al, 0 /* Se termino la fuente? */
		jne strcmp_test2 /* si no, saltar */

		cmp ah, 0 /* Tambien se termino el destino? */
		je strcmp_are_equal /* Las dos cadenas son iguales! */


		strcmp_test2:
		 	/* Las cadenas no se han terminado, comparar sus caracteres */
			cmp al, ah
			je strcmp_loop
		mov WORD PTR [bp - 2], ax /* Son diferentes, almacenar los caracteress*/

	strcmp_are_equal:

	popa

	mov ax, WORD PTR [bp - 2] /* El valor de retorno */

	mov sp, bp /* Cerrar el marco de pila */
	pop bp
	ret


